@@ -23,7 +23,7 @@
 #include "bewegung.h"
 #include "server.h"
 
-static char sccsid[] = "@(#)bewegung.c	1.96 5/26/94";
+static char sccsid[] = "@(#)bewegung.c	1.97 6/12/94";
 
 
 /* Schrittweite der Spieler (Default: 1/8 der Breite eines Feldes) */
@@ -1974,6 +1974,14 @@ void *paket;
 		memcpy(&alle_schuesse[spieler_nr].pos,
 			&alle_spieler[spieler_nr].pos, sizeof(struct position));
 
+		/* Schuss zur Nase bewegen */
+		SUBX(alle_schuesse[spieler_nr].pos,
+			sintab[schuss_blick[spieler_nr] * (TRIGANZ / WINKANZ)] *
+			(KUGELRAD - SCHUSSRAD - 1) / TRIGFAKTOR);
+		SUBY(alle_schuesse[spieler_nr].pos,
+			costab[schuss_blick[spieler_nr] * (TRIGANZ / WINKANZ)] *
+			(KUGELRAD - SCHUSSRAD - 1) / TRIGFAKTOR);
+
 		/* Ereignis merken */
 		ereignisse[spieler_nr][SCHUSS_LOS_ERG] = 1;
 	}
