@@ -28,7 +28,7 @@
 #include "netzwerk.h"
 #include "spiel.h"
 
-static char sccsid[] = "@(#)client.c	1.37 5/17/94";
+static char sccsid[] = "@(#)client.c	1.39 6/27/94";
 
 
 /* Breite und Laenge des aktuellen Spielfeldes: */
@@ -561,12 +561,11 @@ char **argv;
 			argv[0]);
 		exit(1);
 	}
-	/* Verbindung zum angegebenen Server aufbauen
-	   (argv[1] kann NULL sein) */
-	verbindung_aufbauen(argv[1]);
+	/* Verbindung zum angegebenen Server aufbauen */
+	verbindung_aufbauen(argc < 2 ? NULL : argv[1]);
 
 	/* Spielparameter / Labyrinth empfangen */
-	spielparameter_empfangen("iMaze xview client JC/HUK 1.0",
+	spielparameter_empfangen("iMaze xview client JC/HUK 1.1",
 		&feldbreite, &feldlaenge, &spielfeld);
 
 	/* Spielfeld drehen */
