@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#include <signal.h>
 #include <fcntl.h>
 #include <netdb.h>
 #include <sys/types.h>
@@ -21,12 +22,15 @@
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#ifdef _AIX
+#include <sys/select.h>
+#endif
 
 #include "speicher.h"
 #include "server_netz.h"
 #include "system.h"
 
-static char sccsid[] = "@(#)ip_server.c	1.26 5/26/94";
+static char sccsid[] = "@(#)ip_server.c	1.40 7/12/94";
 
 
 /* Default-TCP-Port des Servers */
@@ -50,6 +54,9 @@ struct verbindung
 	                  Kommunikation laeuft (nur fuer den Prolog) */
 	int auf_spiel; /* Flag, ob die Verbindung vom Prolog auf das Spiel
 	                  umgeschaltet wurde */
+	int original;  /* Flag, ob dieser Deskriptor das Original ist und
+	                  noch nicht als Nachricht an einen anderen Prozess
+	                  gesandt wurde */
 };
 
 /* Struktur, in der eine Nachricht fuer die
@@ -73,6 +80,8 @@ union nachricht
 /* Systemvariable: letzter aufgetretener Fehlercode oder 0 */
 extern int errno;
 
+FILE *log_deskriptor; /* Deskriptor fuer das Log-File */
+
 /* im Server-Prozess: */
 static int server_prolog_socket; /* Socket, auf dem die Prolog-Verbindungen
                                     der Clients entgegengenommen werden */
@@ -88,6 +97,21 @@ static void *init_verbindung = NULL; /* Verbindung, auf der der Prozess
                                         den Client initialisiert */
 
 
+/*
+** prozess_ende
+**  wird aufgerufen, wenn ein Prozess stirbt; vermeidet Zombies
+**
+** Parameter:
+**  signum: Dummy-Parameter
+*/
+void prozess_ende(signum)
+int signum;
+{
+	/* Statuswerte der toten Prozesse abfragen */
+	while (waitpid(-1, NULL, WNOHANG) > 0);
+}
+
+
 /*
 ** fehler_text
 **  ermittelt den zur Systemvariable errno passenden Fehlertext
@@ -170,6 +194,32 @@ struct sockaddr_in *adresse;
 }
 
 
+/*
+** verbindungsname
+**  wandelt die Adresse einer Verbindung in einen String um
+**
+** Parameter:
+**  verbindung: die Verbindung, deren Adresse umgewandelt werden soll
+**
+** Rueckgabewert:
+**  String, der die Adresse beschreibt
+**
+** Seiteneffekte:
+**  der Strings wird jedesmal ueberschrieben
+*/
+static char *verbindungsname(verbindung)
+struct verbindung *verbindung;
+{
+	static char name[40];
+
+	sprintf(name, "%s:%d/%s", inet_ntoa(verbindung->adresse.sin_addr),
+		ntohs(verbindung->adresse.sin_port),
+		verbindung->auf_spiel ? "udp" : "tcp");
+
+	return name;
+}
+
+
 /* bis hier lokaler Teil                       */
 /***********************************************/
 /* ab hier globaler Teil                       */
@@ -312,10 +362,11 @@ void **verbindungen, **paket;
 			&paket_adresse_laenge);
 
 		/* Fehler? */
-		if (*paket_laenge < 0)
+		if (*paket_laenge <= 0)
 		{
 			/* kein Paket mehr bereit? */
-			if (errno == EWOULDBLOCK)
+			if (errno == EWOULDBLOCK || errno == EAGAIN ||
+				*paket_laenge == 0)
 				return -1;
 
 			/* bei anderem Fehler Programm mit Fehlermeldung beenden */
@@ -359,6 +410,13 @@ void **verbindungen, **paket;
 void verbindung_freigeben(verbindung)
 void *verbindung;
 {
+	/* mitloggen */
+	if (((struct verbindung *)verbindung)->auf_spiel &&
+		((struct verbindung *)verbindung)->original &&
+		log_deskriptor != NULL)
+		fprintf(log_deskriptor, "%s: player has left on %s\n",
+			uhrzeit(), verbindungsname(verbindung));
+
 	/* ist dies der zugehoerige Initialisierungs-Prozess? */
 	if (verbindung == init_verbindung)
 	{
@@ -396,7 +454,8 @@ void *verbindung_annehmen()
 		(struct sockaddr *)&client_adresse, &client_adresse_laenge)) < 0)
 	{
 		/* keine Verbindung bereit? */
-		if (errno == EINTR || errno == EWOULDBLOCK)
+		if (errno == EINTR || errno == EWOULDBLOCK ||
+			errno == EAGAIN)
 			return NULL;
 
 		/* bei anderem Fehler Programm mit Fehlermeldung beenden */
@@ -419,6 +478,11 @@ void *verbindung_annehmen()
 			   Initialisierungs-Prozess ist */
 			init_verbindung = verbindung;
 
+			/* mitloggen */
+			if (log_deskriptor != NULL)
+				fprintf(log_deskriptor, "%s: connection from %s\n",
+					uhrzeit(), verbindungsname(verbindung));
+
 			/* Verbindungs-Deskriptor zurueckgeben */
 			return verbindung;
 
@@ -447,6 +511,11 @@ void *verbindung_annehmen()
 void verbindung_abbrechen(verbindung)
 void *verbindung;
 {
+	/* mitloggen */
+	if (log_deskriptor != NULL)
+		fprintf(log_deskriptor, "%s: init failed on %s\n",
+			uhrzeit(), verbindungsname(verbindung));
+
 	/* Verbindung schliessen */
 	close(SOCKET(verbindung));
 
@@ -478,6 +547,8 @@ void *session, *verbindung;
 	int spiel_adresse_laenge;               /* Grosse der IP-Adresse */
 	struct sockaddr_in spiel_adresse;       /* IP-Adresse */
 	struct sockaddr_in *verbindung_adresse; /* temporaerer Zeiger */
+	struct verbindung alte_verbindung;      /* alte Verbindung (fuer
+	                                           das Mitloggen) */
 
 	/* Adresse dieser Seite der Verbindung feststellen */
 	spiel_adresse_laenge = sizeof spiel_adresse;
@@ -507,6 +578,9 @@ void *session, *verbindung;
 	if (close(SOCKET(verbindung)))
 		return 1;
 
+	/* alte Verbindung merken */
+	memcpy(&alte_verbindung, verbindung, sizeof alte_verbindung);
+
 	/* Zeiger auf die IP-Adresse im Verbindungs-Deskriptor */
 	verbindung_adresse = &((struct verbindung *)verbindung)->adresse;
 
@@ -521,6 +595,20 @@ void *session, *verbindung;
 	/* Verbindung ist jetzt auf Spiel geschaltet */
 	((struct verbindung *)verbindung)->auf_spiel = 1;
 
+	/* dieser Deskriptor ist das Original */
+	((struct verbindung *)verbindung)->original = 1;
+
+	/* mitloggen */
+	if (log_deskriptor != NULL)
+	{
+		fprintf(log_deskriptor, "%s: %s", uhrzeit(),
+			verbindungsname(&alte_verbindung));
+		fprintf(log_deskriptor, " switched to %s\n",
+			verbindungsname(verbindung));
+		fprintf(log_deskriptor, "%s: %s plays on session %d/udp\n",
+			uhrzeit(), verbindungsname(verbindung), ((int *)session)[1]);
+	}
+
 	/* kein Fehler aufgetreten */
 	return 0;
 }
@@ -548,6 +636,10 @@ char **argv;
 	char *portname;                    /* Name/Nummer des TCP-Ports */
 	int ueberspringen;                 /* Anzahl bereits ausgewerteter
 	                                      Aufrufparameter */
+	char *log_datei_name;              /* Name des Log-Files */
+
+	log_deskriptor = NULL; /* noch kein Deskriptor vorhanden */
+	log_datei_name = NULL; /* noch kein Name fuer das Log-File */
 
 	/* argv[0] ueberspringen */
 	ueberspringen = 1;
@@ -558,27 +650,54 @@ char **argv;
 	/* Name/Nummer des Ports noch unbekannt */
 	portname = NULL;
 
-	/* bei Aufrufparameter "-f" Wiederverwenden erzwingen */
-	if (*argc > ueberspringen && !strcmp(argv[ueberspringen], "-f"))
-		port_wiederverwenden = 1, ueberspringen++;
-
-	/* bei Aufrufparameter "-p" oder "-fp" Portname/nummer merken */
-	if (*argc > ueberspringen + 1 && (!strcmp(argv[ueberspringen], "-p") ||
-		!strcmp(argv[ueberspringen], "-fp")))
+	/* Kommandozeilenparameter auswerten */
+	while (ueberspringen < *argc && argv[ueberspringen][0] == '-')
 	{
-		/* bei Aufrufparameter "-fp" Wiederverwenden erzwingen */
-		if (!strcmp(argv[ueberspringen], "-fp"))
-			port_wiederverwenden = 1;
+		int i; /* Index fuer argv[ueberspringen] */
 
-		/* Portname/nummer merken */
-		portname = argv[ueberspringen + 1];
-		ueberspringen += 2;
+		for (i = 1; argv[ueberspringen][i] != 0 &&
+			strchr("f", argv[ueberspringen][i]) != NULL; i++);
+
+		if (strchr("lp", argv[ueberspringen][i]) != NULL)
+		{
+			if (argv[ueberspringen][i + 1] == 0 &&
+				ueberspringen == *argc - 1)
+				break;
+		}
+		else if (argv[ueberspringen][i] != 0 || i < 2)
+			break;
+
+		/* argv[ueberspingen] faengt mit "-" an und enthaelt
+		   nur bekannte Optionen; Optionen auswerten */
+		for (i = 1; i > 0 && argv[ueberspringen][i] != 0;)
+			switch (argv[ueberspringen][i++])
+			{
+				case 'f': /* Port wiederverwenden ("force") */
+					port_wiederverwenden = 1;
+					break;
+
+				case 'l': /* mitloggen, wer den Server benutzt */
+					if (argv[ueberspringen][i] == 0)
+						ueberspringen++, i = 0;
+					log_datei_name = argv[ueberspringen] + i;
+					i = 0;
+					break;
+
+				case 'p': /* TCP-Port des Servers festlegen */
+					if (argv[ueberspringen][i] == 0)
+						ueberspringen++, i = 0;
+					portname = argv[ueberspringen] + i;
+					i = 0;
+					break;
+			}
+
+		/* naechste Kommandozeilenparameter */
+		ueberspringen++;
 	}
 
-	/* bei Aufrufparameter "-f" (auch nach "-p")
-	   Wiederverwenden erzwingen */
-	if (*argc > ueberspringen && !strcmp(argv[ueberspringen], "-f"))
-		port_wiederverwenden = 1, ueberspringen++;
+	/* "--" ignorieren */
+	if (ueberspringen < *argc && !strcmp(argv[ueberspringen], "--"))
+		ueberspringen++;
 
 	/* mehr als argv[0] ausgewertet? */
 	if (ueberspringen > 1)
@@ -592,6 +711,28 @@ char **argv;
 			argv[i] = argv[i + ueberspringen];
 	}
 
+	/* Name fuer Log-File angegeben? */
+	if (log_datei_name != NULL)
+	{
+		int tmp_deskriptor; /* temporaerer Deskriptor fuer das Log-File */
+
+		/* bei "-" nach stdout loggen */
+		if (!strcmp(log_datei_name, "-"))
+			log_deskriptor = stdout;
+
+		/* sonst Log-File oeffnen/erzeugen */
+		else if ((tmp_deskriptor = open(log_datei_name,
+			O_WRONLY|O_APPEND|O_CREAT, 0666)) < 0 ||
+			(log_deskriptor = fdopen(tmp_deskriptor, "a")) == NULL)
+		{
+			char *text; /* Fehlertext */
+
+			speicher_belegen((void **)&text, strlen(log_datei_name) + 20);
+			sprintf(text, "Can't open \"%s\":", log_datei_name);
+			netzwerk_fehler(text);
+		}
+	}
+
 	/* IP-Adresse des TCP-Sockets initialisieren */
 	server_adresse.sin_family = AF_INET;
 	server_adresse.sin_addr.s_addr = INADDR_ANY;
@@ -599,7 +740,6 @@ char **argv;
 	/* kein Port angegeben? */
 	if (portname == NULL)
 	{
-#ifndef GETSERVBYNAME_BUG
 		struct servent *service_eintrag; /* Portname/-nummer-Zuordnung */
 
 		/* Portname/-nummer-Zuordnung des Ports "imaze" im Protokoll "tcp"
@@ -608,7 +748,6 @@ char **argv;
 			/* Nummer ist bekannt */
 			server_adresse.sin_port = service_eintrag->s_port;
 		else
-#endif
 			/* Nummer ist unbekannt, Default-Portnummer verwenden */
 			server_adresse.sin_port = htons(DEFAULT_PORT);
 	}
@@ -616,7 +755,6 @@ char **argv;
 	{
 		/* Portname ist angegeben, als Nummer interpretieren und merken */
 		if ((server_adresse.sin_port = htons(atoi(portname))) == 0)
-#ifndef GETSERVBYNAME_BUG
 		{
 			struct servent *service_eintrag; /* Portname/-nummer-
 			                                    Zuordnung */
@@ -639,15 +777,6 @@ char **argv;
 			/* gefundene Nummer uebernehmen */
 			server_adresse.sin_port = service_eintrag->s_port;
 		}
-#else
-		{
-			/* Fehlermeldung: */
-			static char *meldung[] = { "iMaze - Network Error", "",
-					"Ports by name not supported", NULL };
-
-			uebler_fehler(meldung, NULL);
-		}
-#endif
 	}
 
 	/* TCP-Socket erzeugen */
@@ -678,7 +807,20 @@ char **argv;
 
 	/* auf dem TCP-Socket ankommende Verbindungen zulassen */
 	if (listen(server_prolog_socket, 5))
-		netzwerk_fehler("Cant listen on prolog socket:");
+		netzwerk_fehler("Can't listen on prolog socket:");
+
+	/* Handler fuer tote Prozesse installieren */
+	signal(SIGCHLD, prozess_ende);
+
+	/* mitloggen? */
+	if (log_deskriptor != NULL)
+	{
+		/* Deskriptor auf Zeilenpufferung schalten */
+		setvbuf(log_deskriptor, NULL, _IOLBF, BUFSIZ);
+
+		/* Start des Servers mitloggen */
+		fprintf(log_deskriptor, "%s: server started\n", uhrzeit());
+	}
 }
 
 
@@ -713,7 +855,7 @@ void *session_starten()
 	/* UDP-Socket in einen Modus versetzen, in dem das Empfangen eines
 	   Pakets den Prozess nicht blockiert, wenn noch kein Paket bereit
 	   ist, sondern einen Fehlercode erzeugt */
-	if (fcntl(session_spiel_socket, F_SETFL, O_NDELAY))
+	if (deskriptor_nicht_blockieren(session_spiel_socket))
 		netzwerk_fehler("Can't switch session socket to non-blocking mode:");
 
 	/* Sockets fuer die Interprozess-Kommunikation erzeugen */
@@ -737,7 +879,7 @@ void *session_starten()
 
 		/* den Kommunikations-Socket in den nicht blockierenden
 		   Modus schalten */
-		if (fcntl(sockpair[1], F_SETFL, O_NDELAY))
+		if (deskriptor_nicht_blockieren(sockpair[1]))
 			netzwerk_fehler(
 				"Can't switch interprocess socket to non-blocking mode:");
 
@@ -772,7 +914,7 @@ void *session_starten()
 
 		/* den Kommunikations-Socket in den nicht blockierenden
 		   Modus schalten */
-		if (fcntl(sockpair[0], F_SETFL, O_NDELAY))
+		if (deskriptor_nicht_blockieren(sockpair[0]))
 			netzwerk_fehler(
 				"Can't switch interprocess socket to non-blocking mode:");
 
@@ -781,7 +923,12 @@ void *session_starten()
 
 		/* den Kommunikations-Socket und die Portnummer speichern */
 		session[0] = sockpair[0];
-		session[1] = htons(spiel_adresse.sin_port);
+		session[1] = ntohs(spiel_adresse.sin_port);
+
+		/* mitloggen */
+		if (log_deskriptor != NULL)
+			fprintf(log_deskriptor, "%s: session started on %d/udp\n",
+				uhrzeit(), session[1]);
 
 		/* den Session-Deskriptor an den Server-Prozess zurueckgeben */
 		return session;
@@ -851,6 +998,9 @@ int typ;
 			exit(1);
 		else
 			netzwerk_fehler("Can't send interprocess message:");
+
+	/* dieser Deskriptor ist nicht mehr das Original */
+	((struct verbindung *)verbindung)->original = 0;
 }
 
 
@@ -888,10 +1038,11 @@ char *typ;
 		socket = *(int *)session;
 
 	/* Nachricht empfangen */
-	if ((laenge = recv(socket, (void *)&nachricht, sizeof nachricht, 0)) < 0)
+	if ((laenge = recv(socket, (void *)&nachricht, sizeof nachricht, 0)) <= 0)
 	{
 		/* Unterbrechung durch ein Signal oder keine Nachricht vorhanden? */
-		if (errno == EINTR || errno == EWOULDBLOCK)
+		if (errno == EINTR || errno == EWOULDBLOCK ||
+			errno == EAGAIN || laenge == 0)
 			/* keine Nachricht */
 			return 1;
 
@@ -920,6 +1071,7 @@ char *typ;
 	*verbindung = verbindung_init(&nachricht.lang.client_adresse);
 	((struct verbindung *)*verbindung)->auf_spiel =
 		nachricht.lang.auf_spiel;
+	((struct verbindung *)*verbindung)->original = 1;
 	*typ = nachricht.lang.typ;
 
 	/* Nachricht empfangen */
