@@ -29,7 +29,7 @@
 #include "X_grafik.h"
 #include "xv_einaus.h"
 
-static char sccsid[] = "@(#)xv_fenster.c	1.58 5/18/94";
+static char sccsid[] = "@(#)xv_fenster.c	1.63 7/15/94";
 
 
 /* die schwarz-weiss-Daten des Icons: */
@@ -1169,7 +1169,7 @@ char **argv;
 
 	/* Erzeugen des Haupfensters */
 	hauptframe = xv_create(XV_NULL, FRAME,
-		FRAME_LABEL, "iMaze - Version 1.0",
+		FRAME_LABEL, "iMaze - Version 1.1",
 		FRAME_ICON, xv_create(XV_NULL, ICON, /* das Icon */
 			ICON_IMAGE, xv_create(XV_NULL, SERVER_IMAGE,
 				XV_WIDTH, 64,
@@ -1225,13 +1225,16 @@ char **argv;
 			0);
 
 		/* kein Panel-Bereich erwuenscht */
-		xv_destroy(xv_get(karte_frame, FRAME_CMD_PANEL));
+		xv_set(xv_get(karte_frame, FRAME_CMD_PANEL),
+			XV_SHOW, FALSE,
+			0);
 
 		/* Zeichenfeld initialisieren */
 		karte_canvas = xv_create(karte_frame, CANVAS,
 			CANVAS_X_PAINT_WINDOW, TRUE,
 			CANVAS_FIXED_IMAGE, FALSE,
 			CANVAS_RETAINED, FALSE,
+			XV_Y, 0,
 			/* Anhaengen von Struktur mit Daten und Funktion, die beim
 			   Neuzeichnen erforderlich sind */
 			XV_KEY_DATA, 1, &karte_fenster,
@@ -1260,12 +1263,6 @@ char **argv;
 			0);
 		window_fit(karte_frame);
 
-		/* Fenstergroesse anpassen */
-		xv_set(xv_get(karte_frame, FRAME_CMD_PANEL),
-			XV_WIDTH, WIN_EXTEND_TO_EDGE,
-			0);
-		window_fit_height(xv_get(karte_frame, FRAME_CMD_PANEL));
-
 		/* Initialisierung der oben genannten Struktur */
 		karte_fenster.daten = &grundriss;
 		karte_fenster.X_zeichne = X_zeichne_grundriss;
@@ -1296,19 +1293,22 @@ char **argv;
 		rueckspiegel_frame = xv_create(hauptframe, FRAME_CMD,
 			XV_X, xv_get(karte_frame, XV_WIDTH) + 12,
 			XV_Y, 0,
-			FRAME_LABEL, "iMaze: Look Back",
+			FRAME_LABEL, "iMaze: Rear View",
 			FRAME_CMD_PUSHPIN_IN, TRUE,
 			FRAME_SHOW_RESIZE_CORNER, TRUE, /* Groesse veraendern erlaubt */
 			0);
 
 		/* kein Panel-Bereich erwuenscht */
-		xv_destroy(xv_get(rueckspiegel_frame, FRAME_CMD_PANEL));
+		xv_set(xv_get(rueckspiegel_frame, FRAME_CMD_PANEL),
+			XV_SHOW, FALSE,
+			0);
 
 		/* Zeichenfeld initialisieren */
 		rueckspiegel_canvas = xv_create(rueckspiegel_frame, CANVAS,
 			CANVAS_X_PAINT_WINDOW, TRUE,
 			CANVAS_FIXED_IMAGE, FALSE,
 			CANVAS_RETAINED, FALSE,
+			XV_Y, 0,
 			/* Anhaengen von Struktur mit Daten und Funktion, die beim
 			   Neuzeichnen erforderlich sind */
 			XV_KEY_DATA, 1, &rueckspiegel_fenster,
@@ -1352,7 +1352,9 @@ char **argv;
 			0);
 
 		/* kein Panel-Bereich erwuenscht */
-		xv_destroy(xv_get(kompass_frame, FRAME_CMD_PANEL));
+		xv_set(xv_get(kompass_frame, FRAME_CMD_PANEL),
+			XV_SHOW, FALSE,
+			0);
 
 		/* Zeichenfeld initialisieren */
 		kompass_canvas = xv_create(kompass_frame, CANVAS,
@@ -1361,6 +1363,7 @@ char **argv;
 			CANVAS_RETAINED, FALSE,
 			/* Anhaengen von Struktur mit Daten und Funktion, die beim
 			   Neuzeichnen erforderlich sind */
+			XV_Y, 0,
 			XV_KEY_DATA, 1, &kompass_fenster,
 			0);
 
@@ -1407,7 +1410,7 @@ char **argv;
 				XV_KEY_DATA, 1, &karte_fenster,
 				0,
 			MENU_ITEM,
-				MENU_STRING, "Look Back",
+				MENU_STRING, "Rear View",
 				MENU_ACTION_PROC, fenster_knopf,
 				/* zu oeffnender Frame */
 				XV_KEY_DATA, 0, rueckspiegel_frame,
@@ -1701,10 +1704,12 @@ char **argv;
 	}
 
 	/* audio-visuelle Untermalung von Ereignissen initialisieren */
-	xv_ereignisse_init(hauptframe, hauptpanel);
+	if (xv_ereignisse_init(hauptframe, hauptpanel, argc, argv))
+		return 1;
 
 	/* Initialisierung der Eingabe-Abfrage */
-	xv_eingabe_init(hauptframe, hauptpanel);
+	if (xv_eingabe_init(hauptframe, hauptpanel, argc, argv))
+		return 1;
 
 	/* bei der Initialisierung ist kein Fehler aufgetreten */
 	return 0;
